==> designs-1.0.0.tar.gz: designs/modules/designs_field/src/DesignsFieldWrapper.php <==
<?php

namespace Drupal\designs_field;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Entity\ContentEntityFormInterface;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\designs\DesignContentManagerInterface;
use Drupal\designs\DesignManagerInterface;
use Drupal\designs\DesignSettingManagerInterface;
use Drupal\designs\Form\PluginForm;

/**
 * Provides functionality for wrapper field formatters in a design.
 */
class DesignsFieldWrapper implements TrustedCallbackInterface {

  use DependencySerializationTrait;
  use StringTranslationTrait;

  /**
   * The design manager.
   *
   * @var \Drupal\designs\DesignManagerInterface
   */
  protected DesignManagerInterface $designManager;

  /**
   * The design setting manager.
   *
   * @var \Drupal\designs\DesignSettingManagerInterface
   */
  protected DesignSettingManagerInterface $settingManager;

  /**
   * The design content manager.
   *
   * @var \Drupal\designs\DesignContentManagerInterface
   */
  protected DesignContentManagerInterface $contentManager;

  /**
   * DesignsFieldWrapper constructor.
   *
   * @param \Drupal\designs\DesignManagerInterface $designManager
   *   The design manager.
   */
  public function __construct(
    DesignManagerInterface $designManager,
    DesignSettingManagerInterface $settingManager,
    DesignContentManagerInterface $contentManager
  ) {
    $this->designManager = $designManager;
    $this->settingManager = $settingManager;
    $this->contentManager = $contentManager;
  }

  /**
   * Build the form for the field wrapper.
   *
   * @param \Drupal\Core\Field\WidgetInterface|\Drupal\Core\Field\FormatterInterface $plugin
   *   The field formatter/widget plugin.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The field definition.
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The configuration render array.
   */
  public function buildConfigurationForm($plugin, FieldDefinitionInterface $field_definition, array &$form, FormStateInterface $form_state) {
    return [
      '#entity_type' => $field_definition->getTargetEntityTypeId(),
      '#entity_bundle' => $field_definition->getTargetBundle(),
      '#field_name' => $field_definition->getName(),
      '#plugin' => $plugin,
      '#process' => [[$this, 'processConfigurationForm']],
    ];
  }

  /**
   * Process the configuration form to use the plugin form.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The form array.
   */
  public function processConfigurationForm(array $form, FormStateInterface $form_state) {
    $plugin = $form['#plugin'];
    $form['#design_submit'] = [[$this, 'submitConfigurationForm']];

    $settings = $form_state->getValue($form['#parents']);
    if (empty($settings)) {
      $settings = $plugin->getThirdPartySettings('designs_field') ?: [];
    }
    $design_form = new PluginForm(
      $this->designManager,
      $this->settingManager,
      $this->contentManager,
      $settings['design'] ?? '',
      $settings,
      'field_wrapper',
      [
        'type' => $form['#entity_type'],
        'bundle' => $form['#entity_bundle'],
        'field' => $form['#field_name'],
      ],
    );

    $form = $design_form->buildForm($form, $form_state);
    $form['#title'] = $this->t('Wrapper Design');

    $form['#element_validate'][] = [$this, 'validateConfigurationForm'];
    return $form;
  }

  /**
   * Validation for ::buildConfigurationForm().
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function validateConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['#form_handler']->validateForm($form, $form_state);
  }

  /**
   * Submission for ::buildConfigurationForm().
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function submitConfigurationForm(array $form, FormStateInterface $form_state) {
    $trigger_element = $form_state->getTriggeringElement();
    $design_parents = $trigger_element['#design_parents'] ?? [];
    $element = NestedArray::getValue($form, $design_parents);
    if (isset($element['#form_handler'])) {
      $element['#form_handler']->submitForm($element, $form_state);
    }
  }

  /**
   * Build the summary for the field wrapper extras.
   *
   * @param \Drupal\Core\Field\WidgetInterface|\Drupal\Core\Field\FormatterInterface $plugin
   *   The field formatter/widget plugin.
   * @param array $summary
   *   The list of summary items.
   *
   * @return array
   *   The updated summary.
   */
  public function buildSummary($plugin, &$summary) {
    $settings = $plugin->getThirdPartySettings('designs_field');
    if (!empty($settings['design'])) {
      if ($this->designManager->hasDefinition($settings['design'])) {
        $design = $this->designManager->getDefinition($settings['design']);
        $summary[] = $this->t('Wrapper Design: @label', ['@label' => $design->getLabel()]);
      }
      else {
        $summary[] = $this->t('Wrapper Design: @label', ['@label' => $this->t('Missing design')]);
      }
    }
    return $summary;
  }

  /**
   * Modifies the built entity render array.
   *
   * @param array $build
   *   The build render array.
   * @param array $context
   *   The display build context.
   */
  public function entityDisplayBuildAlter(array &$build, $context) {
    $this->entityViewAlter($build, $context['entity']);
  }

  /**
   * Modifies the built entity render array.
   *
   * @param array $build
   *   The build render array.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   */
  public function entityViewAlter(array &$build, EntityInterface $entity) {
    // Process each field for the third party settings.
    foreach (Element::children($build) as $field_name) {
      if (empty($build[$field_name]['#third_party_settings']['designs_field']['design'])) {
        continue;
      }

      // Only preform the change just before rendering.
      $build[$field_name]['#pre_render'][] = [static::class, 'preRenderFormatter'];
    }
  }

  /**
   * Modifies the built entity form render array.
   *
   * @param array $form
   *   The build render array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function entityFormAlter(array &$form, FormStateInterface $form_state) {
    $form_object = $form_state->getFormObject();
    if (!$form_object instanceof ContentEntityFormInterface) {
      return;
    }

    // Get the entity form display.
    $entity = $form_object->getEntity();
    $entity_display = EntityFormDisplay::collectRenderDisplay($entity, $form_object->getOperation());

    // Process each of the entity items.
    foreach (Element::children($form) as $field_name) {
      $field = $entity->{$field_name};
      if (!$field) {
        continue;
      }

      // Get the field display for the field name.
      $field_display = $entity_display->getComponent($field_name);
      if (!empty($field_display['third_party_settings']['designs_field']['design'])) {
        $form[$field_name] += [
          '#object' => $entity,
          '#entity_type' => $entity->getEntityTypeId(),
          '#bundle' => $entity->bundle(),
          '#field_name' => $field_name,
          '#third_party_settings' => $field_display['third_party_settings'],
          '#pre_render' => [[static::class, 'preRenderWidget']],
        ];
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['preRenderFormatter', 'preRenderWidget'];
  }

  /**
   * Hide the title when rendering form elements.
   *
   * @param array $element
   *   The render element.
   */
  protected static function hideTitle(&$element) {
    if (isset($element['#title_display'])) {
      $element['#title_display'] = 'hidden';
    }
    foreach (Element::children($element) as $child) {
      self::hideTitle($element[$child]);
    }
  }

  /**
   * Render API: Convert field widget into design.
   *
   * @param array $element
   *   The render element.
   *
   * @return array
   *   The design render element.
   */
  public static function preRenderWidget(array $element) {
    $config = $element['#third_party_settings']['designs_field'];
    /** @var \Drupal\designs\DesignInterface $design */
    $design = \Drupal::service('plugin.manager.design')->createSourcedInstance(
      $config['design'],
      $config,
      'field_wrapper',
      [
        'type' => $element['#entity_type'],
        'bundle' => $element['#bundle'],
        'field' => $element['#field_name'],
      ]
    );

    // Disable the title and description for widgets processed by designs.
    self::hideTitle($element['widget']);

    // Replace widget with design.
    $element['widget'] += array_intersect_key(
      $element,
      array_flip(['#object', '#entity_type', '#bundle', '#field_name'])
    );
    $element['widget'] = $design->build($element['widget']);

    return $element;
  }

  /**
   * Render API: Convert field formatter into design.
   *
   * @param array $element
   *   The render element.
   *
   * @return array
   *   The design render element.
   */
  public static function preRenderFormatter(array $element) {
    $config = $element['#third_party_settings']['designs_field'];
    /** @var \Drupal\designs\DesignInterface $design */
    $design = \Drupal::service('plugin.manager.design')->createSourcedInstance(
      $config['design'],
      $config,
      'field_wrapper',
      [
        'type' => $element['#entity_type'],
        'bundle' => $element['#bundle'],
        'field' => $element['#field_name'],
      ]
    );
    $element += [];
    return ['design' => $design->build($element)];
  }

}

==> designs-1.0.1.tar.gz: designs/modules/designs_field/src/DesignsFieldWrapper.php <==
<?php

namespace Drupal\designs_field;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Entity\ContentEntityFormInterface;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\designs\DesignContentManagerInterface;
use Drupal\designs\DesignManagerInterface;
use Drupal\designs\DesignSettingManagerInterface;
use Drupal\designs\Form\PluginForm;

/**
 * Provides functionality for wrapper field formatters in a design.
 */
class DesignsFieldWrapper implements TrustedCallbackInterface {

  use DependencySerializationTrait;
  use StringTranslationTrait;

  /**
   * The design manager.
   *
   * @var \Drupal\designs\DesignManagerInterface
   */
  protected DesignManagerInterface $designManager;

  /**
   * The design setting manager.
   *
   * @var \Drupal\designs\DesignSettingManagerInterface
   */
  protected DesignSettingManagerInterface $settingManager;

  /**
   * The design content manager.
   *
   * @var \Drupal\designs\DesignContentManagerInterface
   */
  protected DesignContentManagerInterface $contentManager;

  /**
   * DesignsFieldWrapper constructor.
   *
   * @param \Drupal\designs\DesignManagerInterface $designManager
   *   The design manager.
   * @param \Drupal\designs\DesignSettingManagerInterface $settingManager
   *   The design settings manager.
   * @param \Drupal\designs\DesignContentManagerInterface $contentManager
   *   The design content manager.
   */
  public function __construct(
    DesignManagerInterface $designManager,
    DesignSettingManagerInterface $settingManager,
    DesignContentManagerInterface $contentManager
  ) {
    $this->designManager = $designManager;
    $this->settingManager = $settingManager;
    $this->contentManager = $contentManager;
  }

  /**
   * Build the form for the field wrapper.
   *
   * @param \Drupal\Core\Field\WidgetInterface|\Drupal\Core\Field\FormatterInterface $plugin
   *   The field formatter/widget plugin.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The field definition.
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The configuration render array.
   */
  public function buildConfigurationForm($plugin, FieldDefinitionInterface $field_definition, array &$form, FormStateInterface $form_state) {
    return [
      '#entity_type' => $field_definition->getTargetEntityTypeId(),
      '#entity_bundle' => $field_definition->getTargetBundle(),
      '#field_name' => $field_definition->getName(),
      '#plugin' => $plugin,
      '#process' => [[$this, 'processConfigurationForm']],
    ];
  }

  /**
   * Process the configuration form to use the plugin form.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The form array.
   */
  public function processConfigurationForm(array $form, FormStateInterface $form_state) {
    $plugin = $form['#plugin'];
    $form['#design_submit'] = [[$this, 'submitConfigurationForm']];

    $settings = $form_state->getValue($form['#parents']);
    if (empty($settings)) {
      $settings = $plugin->getThirdPartySettings('designs_field') ?: [];
    }
    $design_form = new PluginForm(
      $this->designManager,
      $this->settingManager,
      $this->contentManager,
      $settings['design'] ?? '',
      $settings,
      'field_wrapper',
      [
        'type' => $form['#entity_type'],
        'bundle' => $form['#entity_bundle'],
        'field' => $form['#field_name'],
      ],
    );

    $form = $design_form->buildForm($form, $form_state);
    $form['#title'] = $this->t('Wrapper Design');

    $form['#element_validate'][] = [$this, 'validateConfigurationForm'];
    return $form;
  }

  /**
   * Validation for ::buildConfigurationForm().
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function validateConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['#form_handler']->validateForm($form, $form_state);
  }

  /**
   * Submission for ::buildConfigurationForm().
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function submitConfigurationForm(array $form, FormStateInterface $form_state) {
    $trigger_element = $form_state->getTriggeringElement();
    $design_parents = $trigger_element['#design_parents'] ?? [];
    $element = NestedArray::getValue($form, $design_parents);
    if (isset($element['#form_handler'])) {
      $element['#form_handler']->submitForm($element, $form_state);
    }
  }

  /**
   * Build the summary for the field wrapper extras.
   *
   * @param \Drupal\Core\Field\WidgetInterface|\Drupal\Core\Field\FormatterInterface $plugin
   *   The field formatter/widget plugin.
   * @param array $summary
   *   The list of summary items.
   *
   * @return array
   *   The updated summary.
   */
  public function buildSummary($plugin, array &$summary) {
    $settings = $plugin->getThirdPartySettings('designs_field');
    if (!empty($settings['design'])) {
      if ($this->designManager->hasDefinition($settings['design'])) {
        $design = $this->designManager->getDefinition($settings['design']);
        $summary[] = $this->t('Wrapper Design: @label', ['@label' => $design->getLabel()]);
      }
      else {
        $summary[] = $this->t('Wrapper Design: @label', ['@label' => $this->t('Missing design')]);
      }
    }
    return $summary;
  }

  /**
   * Modifies the built entity render array.
   *
   * @param array $build
   *   The build render array.
   * @param array $context
   *   The display build context.
   */
  public function entityDisplayBuildAlter(array &$build, array $context) {
    $this->entityViewAlter($build, $context['entity']);
  }

  /**
   * Modifies the built entity render array.
   *
   * @param array $build
   *   The build render array.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   */
  public function entityViewAlter(array &$build, EntityInterface $entity) {
    // Process each field for the third party settings.
    foreach (Element::children($build) as $field_name) {
      if (empty($build[$field_name]['#third_party_settings']['designs_field']['design'])) {
        continue;
      }

      // Only preform the change just before rendering.
      $build[$field_name]['#pre_render'][] = [
        static::class,
        'preRenderFormatter'
      ];
    }
  }

  /**
   * Modifies the built entity form render array.
   *
   * @param array $form
   *   The build render array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function entityFormAlter(array &$form, FormStateInterface $form_state) {
    $form_object = $form_state->getFormObject();
    if (!$form_object instanceof ContentEntityFormInterface) {
      return;
    }

    // Get the entity form display.
    $entity = $form_object->getEntity();
    $entity_display = EntityFormDisplay::collectRenderDisplay($entity, $form_object->getOperation());

    // Process each of the entity items.
    foreach (Element::children($form) as $field_name) {
      $field = $entity->{$field_name};
      if (!$field) {
        continue;
      }

      // Get the field display for the field name.
      $field_display = $entity_display->getComponent($field_name);
      if (!empty($field_display['third_party_settings']['designs_field']['design'])) {
        $form[$field_name] += [
          '#object' => $entity,
          '#entity_type' => $entity->getEntityTypeId(),
          '#bundle' => $entity->bundle(),
          '#field_name' => $field_name,
          '#third_party_settings' => $field_display['third_party_settings'],
          '#pre_render' => [[static::class, 'preRenderWidget']],
        ];
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['preRenderFormatter', 'preRenderWidget'];
  }

  /**
   * Hide the title when rendering form elements.
   *
   * @param array $element
   *   The render element.
   */
  protected static function hideTitle(array &$element) {
    if (isset($element['#title_display'])) {
      $element['#title_display'] = 'hidden';
    }
    foreach (Element::children($element) as $child) {
      self::hideTitle($element[$child]);
    }
  }

  /**
   * Render API: Convert field widget into design.
   *
   * @param array $element
   *   The render element.
   *
   * @return array
   *   The design render element.
   */
  public static function preRenderWidget(array $element) {
    $config = $element['#third_party_settings']['designs_field'];
    /** @var \Drupal\designs\DesignInterface $design */
    $design = \Drupal::service('plugin.manager.design')->createSourcedInstance(
      $config['design'],
      $config,
      'field_wrapper',
      [
        'type' => $element['#entity_type'],
        'bundle' => $element['#bundle'],
        'field' => $element['#field_name'],
      ]
    );

    // Disable the title and description for widgets processed by designs.
    self::hideTitle($element['widget']);

    // Replace widget with design.
    $element['widget'] += array_intersect_key(
      $element,
      array_flip(['#object', '#entity_type', '#bundle', '#field_name'])
    );
    $element['widget'] = $design->build($element['widget']);

    return $element;
  }

  /**
   * Render API: Convert field formatter into design.
   *
   * @param array $element
   *   The render element.
   *
   * @return array
   *   The design render element.
   */
  public static function preRenderFormatter(array $element) {
    $config = $element['#third_party_settings']['designs_field'];
    /** @var \Drupal\designs\DesignInterface $design */
    $design = \Drupal::service('plugin.manager.design')->createSourcedInstance(
      $config['design'],
      $config,
      'field_wrapper',
      [
        'type' => $element['#entity_type'],
        'bundle' => $element['#bundle'],
        'field' => $element['#field_name'],
      ]
    );
    $element += [];
    return ['design' => $design->build($element)];
  }

}
